import 'dart:io';

import 'package:dio/dio.dart';
import 'package:xh_framework/com/xh/networkapi/xh_http_request.dart';

class NetworkException implements Exception {
  NetworkException({this.action, this.message});

  final int action;
  final String message;

  static const int connectionTimeout = 1;
  static const int readTimeout = connectionTimeout + 1;
  static const int writeTimeout = readTimeout + 1;
  static const int serverError = writeTimeout + 1;
  static const int clientError = serverError + 1;
  static const int cancelError = clientError + 1;
  static const int otherError = cancelError + 1;
  static const int fileError = otherError + 1;

  static const String connectionTimeoutMsg = '连接超时，请检查网络';
  static const String readTimeoutMsg = '读取超时，请检查网络';
  static const String writeTimeoutMsg = '写入超时，请检查网络';
  static const String serverErrorMsg = '服务端错误';
  static const String clientErrorMsg = '客户端错误';
  static const String cancelErrorMsg = '取消请求';
  static const String otherErrorMsg = '未知错误';

  static void formatException(e, OnErrorCallback onError) {
    NetworkException exception;
    if (e is DioError) {
      if (e.type == DioErrorType.CONNECT_TIMEOUT) {
        exception = NetworkException(
            action: NetworkException.connectionTimeout,
            message: NetworkException.connectionTimeoutMsg);
      } else if (e.type == DioErrorType.SEND_TIMEOUT) {
        exception = NetworkException(
            action: NetworkException.writeTimeout,
            message: NetworkException.writeTimeoutMsg);
      } else if (e.type == DioErrorType.RECEIVE_TIMEOUT) {
        exception = NetworkException(
            action: NetworkException.readTimeout,
            message: NetworkException.readTimeoutMsg);
      } else if (e.type == DioErrorType.RESPONSE) {
        exception = NetworkException(
            action: NetworkException.serverError,
            message: NetworkException.serverErrorMsg);
      } else if (e.type == DioErrorType.DEFAULT) {
        exception = NetworkException(
            action: NetworkException.clientError,
            message: NetworkException.clientErrorMsg);
      } else if (e.type == DioErrorType.CANCEL) {
        exception = NetworkException(
            action: NetworkException.cancelError,
            message: NetworkException.cancelErrorMsg);
      } else {
        exception = NetworkException(
            action: NetworkException.otherError,
            message: NetworkException.otherErrorMsg);
      }
    } else if (e is FileSystemException) {
      exception = NetworkException(
          action: NetworkException.fileError,
          message: e.message.toString() +
              (e.osError != null ? ":" + e.osError.message : ""));
    } else {
      exception = NetworkException(
          action: NetworkException.otherError,
          message: NetworkException.otherErrorMsg);
    }
    if (onError != null) {
      onError(exception);
    }
  }
}
