import 'dart:async';
import 'package:flutter/material.dart';
import 'package:xh_framework/com/xh/base/xh_list_widget.dart';

import 'xh_refresh_state.dart';

abstract class XHRefreshPageStatefulWidgetState<T extends XHListStatefulWidget,
    Model> extends XHRefreshStatefulWidgetState<T, Model> {
  bool _isLoadMore = false;

  @override
  void initState() {
    super.initState();
    scrollController.addListener(() {
      if (scrollController.position.pixels ==
          scrollController.position.maxScrollExtent) {
        setState(() {
          _isLoadMore = true;
        });
        completer = new Completer();
        pageIndex++;
        onRefresh(pageIndex);
      }
    });
  }

  @override
  void onRefresh(int pageIndex) {
    super.onRefresh(pageIndex);
    setState(() {
      _isLoadMore = true;
    });
  }

  void onRefreshComplete(List<Model> list) {
    if (pageIndex != 1) {
      completer.complete();
      setState(() {
        _isLoadMore = false;
        items.addAll(list);
      });
    } else
      super.onRefreshComplete(list);
  }

  void onRefreshError(Exception e) {
    if (_isLoadMore) {
      double edge = 50.0;
      double offsetFromBottom = scrollController.position.maxScrollExtent -
          scrollController.position.pixels;
      if (offsetFromBottom < edge) {
        scrollController.animateTo(
            scrollController.offset - (edge - offsetFromBottom),
            duration: new Duration(milliseconds: 500),
            curve: Curves.easeOut);
      }
    }
    super.onRefreshError(e);
  }

  @override
  Widget buildLoadMoreIndicator() {
    return new Padding(
      padding: const EdgeInsets.all(8.0),
      child: new Center(
          child: new Opacity(
        opacity: 1.0,
        child: new CircularProgressIndicator(),
      )),
    );
  }

  Widget buildListWidget() {
    return new Offstage(
      offstage: message.isNotEmpty,
      child: ListView.builder(
          controller: scrollController,
          itemCount: items.length + 1,
          itemBuilder: (context, index) {
            if (index == items.length) {
              return buildLoadMoreIndicator();
            }
            return buildList(context, index);
          }),
    );
  }
}
